@@ -565,7 +565,7 @@ module NumRu
 	 else
 	   slicer = __rubber_expansion( slicer )
 	 end
-	 val = val.data if val.is_a?(GPhys)
+	 val = val.data if val.respond_to?(:grid) #.is_a?(GPhys)
 	 @data[*slicer] = val
       end
 
@@ -700,10 +700,10 @@ module NumRu
       for f in VArray::Binary_operators
 	 eval <<-EOS, nil, __FILE__, __LINE__+1
 	 def #{f.delete(".")}(other)
-            if other.is_a?(GPhys)
+            if other.respond_to?(:grid) #.is_a?(GPhys)
 	       other, myself = self.shape_coerce(other)
-               if myself.is_a?(GPhys)
-		 if other.is_a?(GPhys)
+               if myself.respond_to?(:grid) #.is_a?(GPhys)
+		 if other.respond_to?(:grid) #.is_a?(GPhys)
 		   vary = myself.data#{f} other.data
                    newgrid = myself.grid.merge(other.grid)
 		 else
@@ -712,7 +712,7 @@ module NumRu
 		 end
 		 GPhys.new( newgrid, vary )
 	       else
-		 if other.is_a?(GPhys)
+		 if other.respond_to?(:grid) #.is_a?(GPhys)
 		   vary = myself#{f} other.data
 		 else
 		   vary = myself#{f} other
@@ -730,7 +730,7 @@ module NumRu
 	 eval <<-EOS, nil, __FILE__, __LINE__+1
 	 def #{f.delete(".")}(other)
             # returns NArray
-            self.data#{f}(other.is_a?(GPhys) ? other.data : other)
+            self.data#{f}(other.respond_to?(:grid) ? other.data : other)
 	 end
 	 EOS
       end
@@ -814,8 +814,10 @@ module NumRu
 	  loopdims = [loopdims]  # put in an Array (if a single Integer/String)
 	end
 	if loopdims.length == 0
-	  raise ArgumentError, "No loop dimension is specified "+
-	    " -- In that case, you don't need this iterator."
+	  #raise ArgumentError, "No loop dimension is specified "+
+	  #  " -- In that case, you don't need this iterator."
+
+	  return yield(*gphyses)  # trivial case supported just for generality
 	end
 
 	#if loopdims.min<0 || loopdims.max>=gp.rank
@@ -885,7 +887,16 @@ module NumRu
 		  end
 		  grid.set_axis(nm,gphyses[0].axis(nm))
 		}
-		vary = VArray.new( NArray.new(rs.ntype, *grid.shape), rs.data )
+		if rs.data[0..0,false].val.respond_to?(:set_mask)
+                   # DEVELOPPER'S NOTE (2006/08/15 horinout). 
+                   # Here, [0..0,false] is to take the minimum subset,
+                   # and respond_to?(:set_mask) is used to check whether
+                   # the data array is compatible to NArrayMiss
+		  vary = VArray.new(NArrayMiss.new(rs.ntype, *grid.shape), 
+				    rs.data)
+		else
+		  vary = VArray.new(NArray.new(rs.ntype, *grid.shape), rs.data)
+		end
 		results_whole.push( GPhys.new( grid, vary ) )
 	      end
 	    end
@@ -1083,4 +1094,10 @@ if $0 == __FILE__
      [ sub.mean(0) ]  # return obj of block is Array -> concat sub GPhys objs
    end
    p gpz3mn.val, gpz3mn.mean(0).val
+
+   # trivial case (with no loop)
+   gpz3mn, = GPhys.each_along_dims(gpz3, []) do |sub|
+     [ sub.mean(0) ]
+   end
+   p( (gpz3mn - gpz3.mean(0)).abs.max )
 end
