@@ -88,10 +88,10 @@ module NumRu
         rank = 4
         bare_index = [ false ] * rank # will be true if coord var is not found
 
-
         axes = Array.new
         for i in 0...rank
-          axpos = VArrayGrADS.new( file.var(axposnames[i]) )
+          axpos = VArrayGrADS.new( file.var(axposnames[i]) )[0...data.shape_current[i]]
+
           cell_center = true
           cell = false
 
