@@ -292,10 +292,11 @@ It is intended to behave as a correspondent of a NetCDFVar object.
 =end
 
 require "date"
-#require "narray"
 require "narray_miss"
+require "numru/gphys/attribute"
 
-class GrADS_Gridded
+module NumRu
+ class GrADS_Gridded
 
   class << self
     alias open new
@@ -525,7 +526,20 @@ EOS
   end
 
   def put(ary)
-    ary = ary.to_na.hton if ( @options["big_endian"] )
+
+    if( (@options["big_endian"] || @options["cray_32bit_ieee"]) &&
+        @options["little_endian"] )
+      raise "endian specification error"
+    end
+
+    if( @options["big_endian"] || @options["cray_32bit_ieee"] )
+      ary = ary.to_na.hton
+    elsif( @options["little_endian"] )
+      ary = ary.to_na.htov
+    elsif( @options["byteswapped"] )
+      ary = ary.to_na.swap_byte
+    end
+
 #    if File.exist?(@dset)
 #      print "overwrite "+@dset+" (y/n)? "
 #      ans = gets[0].chr
@@ -542,6 +556,7 @@ EOS
 
 #  def get(name, level, time)
   def get(name, level, time, lonlat=nil)
+
     if ( @options["template"] )
 #      time_a = get_dim(@dimensions[3]) # julian day(integer) + time of day
       time_a = get_dim(@dimensions[3])
@@ -588,11 +603,15 @@ EOS
     @x_len = @dimensions[0][:len]
     @y_len = @dimensions[1][:len]
 
+    if( (@options["big_endian"] || @options["cray_32bit_ieee"]) &&
+        @options["little_endian"] )
+      raise "endian specification error"
+    end
+
     if(lonlat)
       if !(lonlat.is_a?(Array) || lonlat.is_a?(NArray))
         raise "lonlat must be given Array or NArray"
       end
-#      p lonlat
       lon_str = lonlat[0]
       lon_end = lonlat[1]
       lat_str = lonlat[2]
@@ -602,11 +621,29 @@ EOS
         @datafile.pos = start_byte + @y_len*lon_str*@map[name][:byte]
         ary = NArray.to_na(@datafile.read(@y_len*(lon_end-lon_str+1)*@map[name][:byte]), 
                            @map[name][:type], @y_len, lon_end-lon_str+1)
+
+        if( @options["big_endian"] || @options["cray_32bit_ieee"] )
+          ary = ary.ntoh
+        elsif( @options["little_endian"] )
+          ary = ary.vtoh
+        elsif( @options["byteswapped"] )
+          ary = ary.swap_byte
+        end
+
         ary = ary[lat_str..lat_end,true].transpose
       else
         @datafile.pos = start_byte + @x_len*lat_str*@map[name][:byte]
         ary = NArray.to_na(@datafile.read(@x_len*(lat_end-lat_str+1)*@map[name][:byte]), 
                            @map[name][:type], @x_len, lat_end-lat_str+1)
+
+        if( @options["big_endian"] || @options["cray_32bit_ieee"] )
+          ary = ary.ntoh
+        elsif( @options["little_endian"] )
+          ary = ary.vtoh
+        elsif( @options["byteswapped"] )
+          ary = ary.swap_byte
+        end
+
         ary = ary[lon_str..lon_end,true]
       end
     else
@@ -614,14 +651,16 @@ EOS
 #      ary = NArray.to_na(@datafile.read(@xy*@map[name][:byte]), @map[name][:type],
       ary = NArray.to_na(@datafile.read(@x_len*@y_len*@map[name][:byte]), @map[name][:type],
                          @dimensions[0][:len],@dimensions[1][:len])
-      ary = ary.transpose if ( @map[name][:xytranspose] )
-    end
 
-    ary = ary.ntoh if ( @options["big_endian"] || @options["cray_32bit_ieee"] )
-    ary = ary.vtoh if ( @options["little_endian"] )
-    if( (@options["big_endian"] || @options["cray_32bit_ieee"]) &&
-        @options["little_endian"] )
-      raise "endian specification error"
+      if( @options["big_endian"] || @options["cray_32bit_ieee"] )
+        ary = ary.ntoh
+      elsif( @options["little_endian"] )
+        ary = ary.vtoh
+      elsif( @options["byteswapped"] )
+        ary = ary.swap_byte
+      end
+
+      ary = ary.transpose if ( @map[name][:xytranspose] )
     end
 
     @datafile.close
@@ -641,6 +680,7 @@ EOS
       else
         raise "cannot define dimension "+name
       end
+#      var_dim = var_dim[0..length-1] if (length)
       var_dim = var_dim[-1..0] if (name == 'y' && @options["yrev"])
       var_dim = var_dim[-1..0] if (name == 'z' && @options["zrev"])
     else # time
@@ -653,19 +693,15 @@ EOS
         when 'day'
           var_dim = var_dim.indgen!*dim[:increment]+dim[:start]
         when 'month'
-#          date0 = Date.parse(dim[:units]).jd # date of the initial record
           date0 = dim[:startjd] # (julian day+time of day) of the initial record
           var_dim[0] = dim[:start]
           for i in 1..dim[:len]-1
-#            var_dim[i] = ( Date.new1(var_dim[i-1]) >> dim[:increment] ).jd
             var_dim[i] = ( Date.jd(var_dim[i-1]+date0) >> dim[:increment] ).jd - date0
           end
         when 'year'
-#          date0 = Date.parse(dim[:units]).jd # date of the initial record
           date0 = dim[:startjd] # (julian day+time of day) of the initial record
           var_dim[0] = dim[:start]
           for i in 1..dim[:len]-1
-#            var_dim[i] = ( Date.new1(var_dim[i-1]) >> dim[:increment]*12 ).jd
             var_dim[i] = ( Date.jd(var_dim[i-1]+date0) >> dim[:increment]*12 ).jd - date0
           end
         else
@@ -674,6 +710,7 @@ EOS
       else
         raise "cannot define dimension "+name
       end
+#      var_dim = var_dim[0..length-1] if (length)
     end
     return var_dim
   end
@@ -736,6 +773,8 @@ EOS
         else
           raise "Invalid line: "+line
         end
+      when /^\s*BYTESWAPPED\s*$/i
+        @options["byteswapped"] = true
       when /^\s*OPTIONS\s*(\S+.*)$/i
         if ($1)
 #         $1.split.each{ |opt|
@@ -809,17 +848,10 @@ EOS
             # idim = 3 --- time
             if (dim[:flag] =~ /LINEAR/i)
               start,increment= spec.split
-#              julian_day = parse_starttime(start)
-#              dim[:start] = julian_day
-#              dim[:start_units] = 'days since '+Date.new1(0.0).to_s+' 00UTC'
               year,mon,day,hour,min = parse_starttime(start)
               dim[:start] = 0.0
-#              dim[:start_units] = "days since #{year}-#{mon}-#{day} #{hour}:#{min}:0 +0:00"
               dim[:startjd] = Date.new(year,mon,day).jd + hour/24.0 + min/1440.0
               dim[:increment],dim[:increment_units] = parse_timeincrement(increment)
-#              dim[:units] = dim[:increment_units]
-#              dim[:units] = dim[:start_units]
-#              dim[:units] = "days since #{year}-#{mon}-#{day} #{hour}:#{min}:0 +0:00"
               dim[:units] = sprintf("days since #{year}-%02i-%02i %02i:%02iZ",mon,day,hour,min)
             else
               raise "invalid dimension flag(only LINEAR is available for time)"
@@ -922,10 +954,7 @@ EOS
       if (time<0 || time>=@dimensions[3][:len])
         raise "Time #{time} is not in the data period"
       end
-#      iblock = map[:start]+level*map[:zstep]+time*map[:tstep]
-#      str_byte = map[:offset] + @xybytes*iblock
       iblock = map[:start]+level*map[:zstep]+time*map[:tstep]
-#      str_byte = map[:offset] + @xy*iblock
       str_byte = map[:offset] + @dimensions[0][:len] * @dimensions[1][:len] * iblock
       if( @options["sequential"] )
         str_byte += iblock*8 + 4
@@ -1102,10 +1131,10 @@ EOS
     @ctl_dumped = false
   end
   
-end
+ end
 
 
-class GrADSVar
+ class GrADSVar
 
   def initialize(file,varname)
     @varname = varname
@@ -1126,13 +1155,15 @@ class GrADSVar
       end
     end
 
-    @attr = Attribute.new
+    @attr = NumRu::Attribute.new
     if( @idim != -1 )
       @dimensions = [ @ctl.dimensions[@idim] ]
       @rank = 1
       @attr[:name] = @dimensions[0][:name]
       @attr[:long_name] = @dimensions[0][:description]
       @attr[:units] = @dimensions[0][:units]
+
+      @shape = [@dimensions[0][:len]]
     else
       @dimensions = @ctl.dimensions
       @rank = @dimensions.length
@@ -1141,8 +1172,13 @@ class GrADSVar
         if @varname == @ctl.variables[i][:name]
           @attr[:long_name] = @ctl.variables[i][:description]
           @attr[:nlev] = @ctl.variables[i][:nlev].to_s
+
+          @shape = [@dimensions[0][:len],@dimensions[1][:len],
+                    @dimensions[2][:len],@dimensions[3][:len]]
+          @shape[2] = @ctl.variables[i][:nlev].to_i if( @ctl.variables[i][:nlev] )
+
           @attr[:missing_value] = NArray.sfloat(1).fill!(@ctl.undef)
-	  found = true
+          found = true
         end
       end
       raise "variable #{@varname} is not found" if !found
@@ -1232,26 +1268,29 @@ class GrADSVar
     @ctl.ctlfile
   end
 
-  def shape_ul0
-    sh = []
-    @dimensions.each{|dim|
-#      if d.unlimited? then
-#        sh.push(0)
-#      else
-        sh.push(dim[:len])
-#      end
-    }
-    sh
-  end
+#  def shape_ul0
+#    sh = []
+#    @dimensions.each{|dim|
+##      if d.unlimited? then
+##        sh.push(0)
+##      else
+#        sh.push(dim[:len])
+##      end
+#    }
+#    sh
+#  end
 
   def shape_current
-    sh = []
-    @dimensions.each{|dim|
-      sh.push(dim[:len])
-    }
-    sh
+#    sh = []
+#    @dimensions.each{|dim|
+#      sh.push(dim[:len])
+#    }
+#    sh
+    @shape
   end
 
+  alias shape_ul0 shape_current
+
   def scaled_put(var,hash=nil)
     raise "scaled_put not supported"
   end
@@ -1265,7 +1304,6 @@ class GrADSVar
   end
 
   def get(hash=nil)
-#    if @rank == 1 && @varname == @dimensions[0][:name]
     if @idim != -1
       na = @ctl.get_dim(@dimensions[0])
       if hash == nil
@@ -1305,10 +1343,14 @@ class GrADSVar
         end
       end
 
-      idx_x = NArray.int(@ctl.dimensions[0][:len]).indgen!
-      idx_y = NArray.int(@ctl.dimensions[1][:len]).indgen!
-      idx_z = NArray.int(@ctl.dimensions[2][:len]).indgen!
-      idx_t = NArray.int(@ctl.dimensions[3][:len]).indgen!
+#      idx_x = NArray.int(@ctl.dimensions[0][:len]).indgen!
+#      idx_y = NArray.int(@ctl.dimensions[1][:len]).indgen!
+#      idx_z = NArray.int(@ctl.dimensions[2][:len]).indgen!
+#      idx_t = NArray.int(@ctl.dimensions[3][:len]).indgen!
+      idx_x = NArray.int(@shape[0]).indgen!
+      idx_y = NArray.int(@shape[1]).indgen!
+      idx_z = NArray.int(@shape[2]).indgen!
+      idx_t = NArray.int(@shape[3]).indgen!
       str_x = idx_x[h_sta[0]]; end_x = idx_x[h_end[0]]
       str_y = idx_y[h_sta[1]]; end_y = idx_y[h_end[1]]
       str_z = idx_z[h_sta[2]]; end_z = idx_z[h_end[2]]
@@ -1317,12 +1359,6 @@ class GrADSVar
       na = NArrayMiss.new(vartype,end_x-str_x+1,end_y-str_y+1,end_z-str_z+1,end_t-str_t+1)
       for time in str_t..end_t
         for lev in str_z..end_z
-#          na_xy = @ctl.get(@varname,lev,time)
-##          if time == str_t && lev == str_z
-##            na = NArrayMiss.new(vartype,end_x-str_x+1,end_y-str_y+1,end_z-str_z+1,end_t-str_t+1)
-##          end
-#          mask = ( na_xy[str_x..end_x,str_y..end_y].ne @ctl.undef )
-#          na[true,true,lev-str_z,time-str_t] = NArrayMiss.to_nam(na_xy[str_x..end_x,str_y..end_y],mask)
           na_xy = @ctl.get(@varname,lev,time,[str_x,end_x,str_y,end_y])
           mask = ( na_xy.ne @ctl.undef )
           na[true,true,lev-str_z,time-str_t] = NArrayMiss.to_nam(na_xy,mask)
@@ -1391,7 +1427,9 @@ class GrADSVar
            na_tmp = self[*a_new]
            if n==0 then
              k = at
-             a[0..at-1].each{|x| if x.is_a?(Fixnum) then k -= 1 end}
+             if at > 0
+               a[0..at-1].each{|x| if x.is_a?(Fixnum) then k -= 1 end}
+             end
              shape_tmp = na_tmp.shape
              shape_tmp[k] = i.length
              na = na_tmp.class.new(na_tmp.typecode,*shape_tmp)
@@ -1488,95 +1526,14 @@ class GrADSVar
      end
    end
 
-=begin
-   def [](*a)  # copied from NetCDFVar class
-     first = Array.new
-     last = Array.new
-     stride = Array.new
-     set_stride = false
-     a.each{|i|
-       if(i.is_a?(Fixnum))
-         first.push(i)
-         last.push(i)
-         stride.push(1)
-       elsif(i.is_a?(Range))
-         first.push(i.first)
-         last.push(i.exclude_end? ? i.last-1 : i.last)
-         stride.push(1)
-       elsif(i.is_a?(Hash))
-         r = (i.to_a[0])[0]
-         s = (i.to_a[0])[1]
-         if ( !( r.is_a?(Range) ) || ! ( s.is_a?(Integer) ) )
-            raise TypeError, "Hash argument must be {a_Range, step}"
-         end
-         first.push(r.first) 
-         last.push(r.exclude_end? ? r.last-1 : r.last)
-         stride.push(s)
-         set_stride = true
-       elsif(i.is_a?(TrueClass))
-         first.push(0)
-         last.push(-1)
-         stride.push(1)
-       end
-     }
-
-     if(set_stride)
-       na = self.get({"start"=>first, "end"=>last, "stride"=>stride})
-     else
-       na = self.get({"start"=>first, "end"=>last})
-     end
-     shape = na.shape
-     (a.length-1).downto(0){ |i|
-         shape.delete_at(i) if a[i].is_a?(Fixnum)
-     }
-     na.reshape!( *shape )
-     na
-   end
-
-   def []=(*a)  # copied from NetCDFVar class
-     val = a.pop
-     first = Array.new
-     last = Array.new
-     stride = Array.new
-     set_stride = false
-     a.each{|i|
-       if(i.is_a?(Fixnum))
-         first.push(i)
-         last.push(i)
-         stride.push(1)
-       elsif(i.is_a?(Range))
-         first.push(i.first)
-         last.push(i.exclude_end? ? i.last-1 : i.last)
-         stride.push(1)
-       elsif(i.is_a?(Hash))
-         r = (i.to_a[0])[0]
-         s = (i.to_a[0])[1]
-         if ( !( r.is_a?(Range) ) || ! ( s.is_a?(Integer) ) )
-            raise ArgumentError, "Hash argument must be {first..last, step}"
-         end
-         first.push(r.first) 
-         last.push(r.exclude_end? ? r.last-1 : r.last)
-         stride.push(s)
-         set_stride = true
-       elsif(i.is_a?(TrueClass))
-         first.push(0)
-         last.push(-1)
-         stride.push(1)
-       end
-     }
-
-     if(set_stride)
-        self.put(val, {"start"=>first, "end"=>last, "stride"=>stride})
-     else
-        self.put(val, {"start"=>first, "end"=>last})
-     end
-   end
-=end
+ end
 
 end
 
 if __FILE__ == $0 then
 
+  include NumRu
+
   begin
     gr = GrADS_Gridded.open("../../testdata/T.jan.ctl")
   rescue
@@ -1589,10 +1546,6 @@ if __FILE__ == $0 then
   p gr.get_dim(gr.dimensions[2])
   p gr.get('T',0,0)
 
-
-  require "numru/gphys/attribute"
-  include NumRu
-
   begin
     grvar = GrADSVar.new("../../testdata/T.jan.ctl","T")
   rescue
