@@ -165,7 +165,7 @@ module NumRu
       rescue
 	#if $VERBOSE
 	$stderr.print(
-                   "WARNING: incompatible units: #{u1.to_s} - #{u2.to_s}\n")
+                   "WARNING: incompatible units: #{units.to_s} - #{to_units.to_s}\n")
 	#end   # warn in Ruby 1.8
 	self
       end
@@ -221,6 +221,10 @@ module NumRu
       end
     end
     
+    def **(other)
+      UNumeric.new( val**other, units**other )
+    end
+
     def abs
       UNumeric.new( val.abs, units )
     end
@@ -238,7 +242,7 @@ module NumRu
     end
 
     def /(other)
-      self * (1.0/other)   # not efficient  --> Rewrite later!
+      self * (other**(-1))   # not efficient  --> Rewrite later!
     end
 
     LogicalOps = [">",">=","<","<=","==","==="]
@@ -262,17 +266,37 @@ module NumRu
       EOS
     }
 
-    Nondim_Math_funcs = ["exp","log","log10","log2","sin","cos","tan",
-	    "sinh","cosh","tanh","asin","acos","atan","asinh","acosh",
-	    "atanh","csc","sec","cot","csch","sech","coth","acsc","asec",
-	    "acot","acsch","asech","acoth"]
-    Nondim_Math_funcs.each{ |f|
+    Math_funcs_nondim = ["exp","log","log10","log2","sin","cos","tan",
+	    "sinh","cosh","tanh","asinh","acosh",
+	    "atanh","csc","sec","cot","csch","sech","coth",
+	    "acsch","asech","acoth"]
+    Math_funcs_nondim.each{ |f|
       eval <<-EOS, nil, __FILE__, __LINE__+1
         def #{f}
-          Math.#{f}(val)
+          UNumeric.new( Math.#{f}(val), Units.new('1') )
         end
       EOS
     }
+    Math_funcs_radian = ["asin","acos","atan","acsc","asec","acot"]
+    Math_funcs_radian.each{ |f|
+      eval <<-EOS, nil, __FILE__, __LINE__+1
+        def #{f}
+          UNumeric.new( Math.#{f}(val), Units.new('rad') )
+        end
+      EOS
+    }
+
+    def atan2(other)
+      case other
+      when Numeric
+	UNumeric.new( Math.atan2(val, other), Units.new('rad') )
+      when UNumeric
+	UNumeric.new( Math.atan2(val, other.val), Units.new('rad') )
+      else
+	c_me, c_other = other.coerce(self)
+	c_me.atan2(c_other)
+      end
+    end
 
     def sqrt
       UNumeric.new( Math.sqrt(val), units**Rational(1,2) )
@@ -284,7 +308,7 @@ end   # module NumRu
 ######################################
 if $0 == __FILE__
 
-  require "numru/gphys/gphys"
+  require "narray"
 
   include NumRu
   a = UNumeric[ 10.0, Units['m/s'] ]
@@ -300,12 +324,17 @@ if $0 == __FILE__
   p a*7
   p -a
   p a-b, a-1000, a/100
+  p a.log, a.sin
+  p UNumeric[1.0,Units['1']].atan2( UNumeric[1.0,Units['1']] )
 
   print "\n** Section 2 **\n"
   p a > 1
   p 1 > a
 
   print "\n** Section 3 **\n"
+
+  require "numru/gphys/varray"
+
   na = NArray.float(4).indgen
   va = VArray.new( na )
   vb = a + va
